/*
rndinit.c  Version 1.3.0
Initialize random number generator seed to date, time,
& clock ticks. Resulting seed is unpredictable.
Copyright (C) 1999-2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Usage:
    	unsigned char *rndSeed;
    	rndSeed = (unsigned char *) rndinit();
*/

/*
User subroutine calls:
	void rnd(unsigned char *rndSeed);

System subroutine calls:
	times
	time
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/times.h>
#define SEEDLEN 16

int rnd(unsigned char *rndSeed);

unsigned char *rndinit(void)
	{
	int i,j;
	unsigned char *rndSeed,*r;
	time_t now,*p;
	clock_t clk,*q;
	struct tms t;
	/* allocate memory for seed */
	rndSeed = (unsigned char *) malloc(SEEDLEN+16);
	if (rndSeed == NULL)
	   {
	   fprintf(stderr,"rndinit: out of memory "
	      "allocating rndSeed\n");
	   return(NULL);
	   } /* out of memory */
	/* get clock ticks since boot */
	clk = times(&t);
	/* get date & time */
	time(&now);
	r = rndSeed;
	/* initialize seed to zeros */
	for (i=0;i<SEEDLEN;i++) *r++ = '\0';
	/* fill seed with unique data */
	r = rndSeed;
	for (i=0;i<2;i++)
	   {
	   p = &now;	/* date & time */
	   for (j=0;j<4;j++) *r++ = *p++;
	   q = &clk;	/* clock ticks @ 100/sec */
	   for (j=0;j<4;j++) *r++ = *q++;
	   } /* for each 8 bytes of seed */
	*rndSeed |= 1;	/* seed must be odd */
	/* make seed random */
	for (i=0;i<4096;i++) rnd(rndSeed);
	return(rndSeed);
	} /* rndinit */
